@@ -18,7 +18,13 @@ input_ = av.open(
 )
 output = av.open("out.mkv", "w")
 
-output_stream = output.add_stream("h264", rate=30)
+# Prefer x264, but use Apple hardware if not available.
+try:
+    encoder = av.Codec("libx264", "w").name
+except av.FFmpegError:
+    encoder = "h264_videotoolbox"
+
+output_stream = output.add_stream(encoder, rate=30)
 output_stream.width = input_.streams.video[0].width
 output_stream.height = input_.streams.video[0].height
 output_stream.pix_fmt = "yuv420p"
