@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import io
 import math
 from fractions import Fraction
@@ -409,9 +411,11 @@ class TestMaxBFrameEncoding(TestCase):
             assert actual_max_b_frames <= max_b_frames
 
 
-def encode_frames_with_qminmax(frames: list, shape: tuple, qminmax: tuple) -> int:
+def encode_frames_with_qminmax(
+    frames: list[VideoFrame], shape: tuple[int, int, int], qminmax: tuple[int, int]
+) -> int:
     """
-    Encode a video with the given quantiser limits, and return how many enocded
+    Encode a video with the given quantiser limits, and return how many encoded
     bytes we made in total.
 
     frames: the frames to encode
