@@ -7,16 +7,11 @@ import pytest
 
 import av
 from av import VideoFrame
+from av.frame import Frame
+from av.video.frame import supported_np_pix_fmts
 from av.video.reformatter import ColorRange, Colorspace, Interpolation
 
-from .common import (
-    TestCase,
-    assertImagesAlmostEqual,
-    assertNdarraysEqual,
-    fate_png,
-    fate_suite,
-    has_pillow,
-)
+from .common import TestCase, assertNdarraysEqual, fate_png, fate_suite
 
 
 def assertPixelValue16(plane, expected, byteorder: str) -> None:
@@ -29,24 +24,21 @@ def assertPixelValue16(plane, expected, byteorder: str) -> None:
         assert view[1] == (expected >> 8 & 0xFF)
 
 
-def test_opaque() -> None:
+def test_frame_duration_matches_packet() -> None:
     with av.open(fate_suite("h264/interlaced_crop.mp4")) as container:
-        video_stream = container.streams.video[0]
+        packet_durations = [
+            (p.pts, p.duration) for p in container.demux() if p.pts is not None
+        ]
+        packet_durations.sort(key=lambda x: x[0])
 
-        ctx = video_stream.codec_context
-        ctx.flags |= av.codec.context.Flags.copy_opaque
+    with av.open(fate_suite("h264/interlaced_crop.mp4")) as container:
+        frame_durations = [
+            (f.pts, f.duration) for f in container.decode(video=0) if f.pts is not None
+        ]
+        frame_durations.sort(key=lambda x: x[0])
 
-        assert video_stream.codec_context.copy_opaque
-
-        for packet_idx, packet in enumerate(container.demux()):
-            packet.opaque = (time.time(), packet_idx)
-            for frame in packet.decode():
-                assert isinstance(frame, av.frame.Frame)
-
-                if frame.opaque is None:
-                    continue
-
-                assert type(frame.opaque) is tuple and len(frame.opaque) == 2
+    assert len(packet_durations) == len(frame_durations)
+    assert all(pd[1] == fd[1] for pd, fd in zip(packet_durations, frame_durations))
 
 
 def test_invalid_pixel_format() -> None:
@@ -132,64 +124,39 @@ def test_memoryview_read() -> None:
     assert mem[:7] == b"0.234xx"
 
 
-class TestVideoFrameImage(TestCase):
-    def setUp(self) -> None:
-        if not has_pillow:
-            pytest.skip()
+def test_opaque() -> None:
+    frame = VideoFrame(640, 480, "rgb24")
+    frame.opaque = 3
+    assert frame.opaque == 3
+    frame.opaque = "a"
+    assert frame.opaque == "a"
 
-    def test_roundtrip(self) -> None:
-        import PIL.Image as Image
+    greeting = "Hello World!"
+    frame.opaque = greeting
+    assert frame.opaque is greeting
 
-        image = Image.open(fate_png())
-        frame = VideoFrame.from_image(image)
-        img = frame.to_image()
-        img.save(self.sandboxed("roundtrip-high.jpg"))
-        assertImagesAlmostEqual(image, img)
+    frame.opaque = None
+    assert frame.opaque is None
 
-    def test_interpolation(self) -> None:
-        import PIL.Image as Image
 
-        image = Image.open(fate_png())
-        frame = VideoFrame.from_image(image)
-        assert frame.width == 330 and frame.height == 330
+def test_interpolation() -> None:
+    container = av.open(fate_png())
+    for _ in container.decode(video=0):
+        frame = _
+        break
 
-        img = frame.to_image(width=200, height=100, interpolation=Interpolation.BICUBIC)
-        assert img.width == 200 and img.height == 100
+    assert frame.width == 330 and frame.height == 330
 
-        img = frame.to_image(width=200, height=100, interpolation="BICUBIC")
-        assert img.width == 200 and img.height == 100
+    img = frame.reformat(width=200, height=100, interpolation=Interpolation.BICUBIC)
+    assert img.width == 200 and img.height == 100
 
-        img = frame.to_image(
-            width=200, height=100, interpolation=int(Interpolation.BICUBIC)
-        )
-        assert img.width == 200 and img.height == 100
+    img = frame.reformat(width=200, height=100, interpolation="BICUBIC")
+    assert img.width == 200 and img.height == 100
 
-    def test_to_image_rgb24(self) -> None:
-        sizes = [(318, 238), (320, 240), (500, 500)]
-        for width, height in sizes:
-            frame = VideoFrame(width, height, format="rgb24")
-
-            # fill video frame data
-            for plane in frame.planes:
-                ba = bytearray(plane.buffer_size)
-                pos = 0
-                for row in range(height):
-                    for i in range(plane.line_size):
-                        ba[pos] = i % 256
-                        pos += 1
-                plane.update(ba)
-
-            # construct expected image data
-            expected = bytearray(height * width * 3)
-            pos = 0
-            for row in range(height):
-                for i in range(width * 3):
-                    expected[pos] = i % 256
-                    pos += 1
-
-            img = frame.to_image()
-            assert img.size == (width, height)
-            assert img.tobytes() == expected
+    img = frame.reformat(
+        width=200, height=100, interpolation=int(Interpolation.BICUBIC)
+    )
+    assert img.width == 200 and img.height == 100
 
 
 def test_basic_to_ndarray() -> None:
@@ -197,19 +164,11 @@ def test_basic_to_ndarray() -> None:
     assert array.shape == (480, 640, 3)
 
 
-def test_to_image_with_dimensions() -> None:
-    if not has_pillow:
-        pytest.skip()
-
-    img = VideoFrame(640, 480, format="rgb24").to_image(width=320, height=240)
-    assert img.size == (320, 240)
-
-
 def test_ndarray_gray() -> None:
     array = numpy.random.randint(0, 256, size=(480, 640), dtype=numpy.uint8)
     for format in ("gray", "gray8"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == "gray"
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -219,17 +178,127 @@ def test_ndarray_gray_align() -> None:
     array = numpy.random.randint(0, 256, size=(238, 318), dtype=numpy.uint8)
     for format in ("gray", "gray8"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == "gray"
         assertNdarraysEqual(frame.to_ndarray(), array)
 
 
+def test_ndarray_gray9be() -> None:
+    array = numpy.random.randint(0, 512, size=(480, 640), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="gray9be")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "gray9be"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0], "big")
+
+
+def test_ndarray_gray9le() -> None:
+    array = numpy.random.randint(0, 512, size=(480, 640), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="gray9le")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "gray9le"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0], "little")
+
+
+def test_ndarray_gray10be() -> None:
+    array = numpy.random.randint(0, 1024, size=(480, 640), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="gray10be")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "gray10be"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0], "big")
+
+
+def test_ndarray_gray10le() -> None:
+    array = numpy.random.randint(0, 1024, size=(480, 640), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="gray10le")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "gray10le"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0], "little")
+
+
+def test_ndarray_gray12be() -> None:
+    array = numpy.random.randint(0, 4096, size=(480, 640), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="gray12be")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "gray12be"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0], "big")
+
+
+def test_ndarray_gray12le() -> None:
+    array = numpy.random.randint(0, 4096, size=(480, 640), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="gray12le")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "gray12le"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0], "little")
+
+
+def test_ndarray_gray14be() -> None:
+    array = numpy.random.randint(0, 16384, size=(480, 640), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="gray14be")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "gray14be"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0], "big")
+
+
+def test_ndarray_gray14le() -> None:
+    array = numpy.random.randint(0, 16384, size=(480, 640), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="gray14le")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "gray14le"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0], "little")
+
+
+def test_ndarray_gray16be() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="gray16be")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "gray16be"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0], "big")
+
+
+def test_ndarray_gray16le() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="gray16le")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "gray16le"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0], "little")
+
+
 def test_ndarray_grayf32() -> None:
     array = numpy.random.random_sample(size=(480, 640)).astype(numpy.float32)
     for format in ("grayf32be", "grayf32le"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -239,7 +308,7 @@ def test_ndarray_grayf32_align() -> None:
     array = numpy.random.random_sample(size=(238, 318)).astype(numpy.float32)
     for format in ("grayf32be", "grayf32le"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -249,7 +318,7 @@ def test_ndarray_rgb() -> None:
     array = numpy.random.randint(0, 256, size=(480, 640, 3), dtype=numpy.uint8)
     for format in ("rgb24", "bgr24"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -259,17 +328,26 @@ def test_ndarray_rgb_align() -> None:
     array = numpy.random.randint(0, 256, size=(238, 318, 3), dtype=numpy.uint8)
     for format in ("rgb24", "bgr24"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
 
 
+def test_ndarray_rgbf32() -> None:
+    array = numpy.random.random_sample(size=(480, 640, 3)).astype(numpy.float32)
+    for format in ("rgbf32be", "rgbf32le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
 def test_ndarray_rgba() -> None:
     array = numpy.random.randint(0, 256, size=(480, 640, 4), dtype=numpy.uint8)
     for format in ("argb", "rgba", "abgr", "bgra"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -279,7 +357,150 @@ def test_ndarray_rgba_align() -> None:
     array = numpy.random.randint(0, 256, size=(238, 318, 4), dtype=numpy.uint8)
     for format in ("argb", "rgba", "abgr", "bgra"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
+        assert frame.width == 318 and frame.height == 238
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_bayer8() -> None:
+    array = numpy.random.randint(0, 256, size=(480, 640), dtype=numpy.uint8)
+    for format in ("bayer_bggr8", "bayer_gbrg8", "bayer_grbg8", "bayer_rggb8"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in supported_np_pix_fmts
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_bayer16() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640), dtype=numpy.uint16)
+    for format in (
+        "bayer_bggr16be",
+        "bayer_bggr16le",
+        "bayer_gbrg16be",
+        "bayer_gbrg16le",
+        "bayer_grbg16be",
+        "bayer_grbg16le",
+        "bayer_rggb16be",
+        "bayer_rggb16le",
+    ):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in supported_np_pix_fmts
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrap() -> None:
+    array = numpy.random.randint(0, 256, size=(480, 640, 4), dtype=numpy.uint8)
+    frame = VideoFrame.from_ndarray(array, format="gbrap")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "gbrap"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrap_align() -> None:
+    array = numpy.random.randint(0, 256, size=(238, 318, 4), dtype=numpy.uint8)
+    frame = VideoFrame.from_ndarray(array, format="gbrap")
+    assert frame.width == 318 and frame.height == 238
+    assert frame.format.name == "gbrap"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrap10() -> None:
+    array = numpy.random.randint(0, 1024, size=(480, 640, 4), dtype=numpy.uint16)
+    for format in ("gbrap10be", "gbrap10le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in supported_np_pix_fmts
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrap10_align() -> None:
+    array = numpy.random.randint(0, 1024, size=(238, 318, 4), dtype=numpy.uint16)
+    for format in ("gbrap10be", "gbrap10le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in supported_np_pix_fmts
+        assert frame.width == 318 and frame.height == 238
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrap12() -> None:
+    array = numpy.random.randint(0, 4096, size=(480, 640, 4), dtype=numpy.uint16)
+    for format in ("gbrap12be", "gbrap12le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in supported_np_pix_fmts
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrap12_align() -> None:
+    array = numpy.random.randint(0, 4096, size=(238, 318, 4), dtype=numpy.uint16)
+    for format in ("gbrap12be", "gbrap12le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in supported_np_pix_fmts
+        assert frame.width == 318 and frame.height == 238
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrap14() -> None:
+    array = numpy.random.randint(0, 16384, size=(480, 640, 4), dtype=numpy.uint16)
+    for format in ("gbrap14be", "gbrap14le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in supported_np_pix_fmts
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrap14_align() -> None:
+    array = numpy.random.randint(0, 16384, size=(238, 318, 4), dtype=numpy.uint16)
+    for format in ("gbrap14be", "gbrap14le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in supported_np_pix_fmts
+        assert frame.width == 318 and frame.height == 238
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrap16() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640, 4), dtype=numpy.uint16)
+    for format in ("gbrap16be", "gbrap16le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in supported_np_pix_fmts
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrap16_align() -> None:
+    array = numpy.random.randint(0, 65536, size=(238, 318, 4), dtype=numpy.uint16)
+    for format in ("gbrap16be", "gbrap16le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in supported_np_pix_fmts
+        assert frame.width == 318 and frame.height == 238
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrapf32() -> None:
+    array = numpy.random.random_sample(size=(480, 640, 4)).astype(numpy.float32)
+    for format in ("gbrapf32be", "gbrapf32le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrapf32_align() -> None:
+    array = numpy.random.random_sample(size=(238, 318, 4)).astype(numpy.float32)
+    for format in ("gbrapf32be", "gbrapf32le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -301,11 +522,31 @@ def test_ndarray_gbrp_align() -> None:
     assertNdarraysEqual(frame.to_ndarray(), array)
 
 
+def test_ndarray_gbrp9() -> None:
+    array = numpy.random.randint(0, 512, size=(480, 640, 3), dtype=numpy.uint16)
+    for format in ("gbrp9be", "gbrp9le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in supported_np_pix_fmts
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_gbrp9_align() -> None:
+    array = numpy.random.randint(0, 512, size=(238, 318, 3), dtype=numpy.uint16)
+    for format in ("gbrp9be", "gbrp9le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in supported_np_pix_fmts
+        assert frame.width == 318 and frame.height == 238
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
 def test_ndarray_gbrp10() -> None:
     array = numpy.random.randint(0, 1024, size=(480, 640, 3), dtype=numpy.uint16)
     for format in ("gbrp10be", "gbrp10le"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -315,7 +556,7 @@ def test_ndarray_gbrp10_align() -> None:
     array = numpy.random.randint(0, 1024, size=(238, 318, 3), dtype=numpy.uint16)
     for format in ("gbrp10be", "gbrp10le"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -325,7 +566,7 @@ def test_ndarray_gbrp12() -> None:
     array = numpy.random.randint(0, 4096, size=(480, 640, 3), dtype=numpy.uint16)
     for format in ("gbrp12be", "gbrp12le"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -335,7 +576,7 @@ def test_ndarray_gbrp12_align() -> None:
     array = numpy.random.randint(0, 4096, size=(238, 318, 3), dtype=numpy.uint16)
     for format in ("gbrp12be", "gbrp12le"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -345,7 +586,7 @@ def test_ndarray_gbrp14() -> None:
     array = numpy.random.randint(0, 16384, size=(480, 640, 3), dtype=numpy.uint16)
     for format in ("gbrp14be", "gbrp14le"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -355,7 +596,7 @@ def test_ndarray_gbrp14_align() -> None:
     array = numpy.random.randint(0, 16384, size=(238, 318, 3), dtype=numpy.uint16)
     for format in ("gbrp14be", "gbrp14le"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -365,7 +606,7 @@ def test_ndarray_gbrp16() -> None:
     array = numpy.random.randint(0, 65536, size=(480, 640, 3), dtype=numpy.uint16)
     for format in ("gbrp16be", "gbrp16le"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -375,7 +616,7 @@ def test_ndarray_gbrp16_align() -> None:
     array = numpy.random.randint(0, 65536, size=(238, 318, 3), dtype=numpy.uint16)
     for format in ("gbrp16be", "gbrp16le"):
         frame = VideoFrame.from_ndarray(array, format=format)
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -399,24 +640,6 @@ def test_ndarray_gbrpf32_align() -> None:
         assertNdarraysEqual(frame.to_ndarray(), array)
 
 
-def test_ndarray_gbrapf32() -> None:
-    array = numpy.random.random_sample(size=(480, 640, 4)).astype(numpy.float32)
-    for format in ("gbrapf32be", "gbrapf32le"):
-        frame = VideoFrame.from_ndarray(array, format=format)
-        assert frame.width == 640 and frame.height == 480
-        assert frame.format.name == format
-        assertNdarraysEqual(frame.to_ndarray(), array)
-
-
-def test_ndarray_gbrapf32_allign() -> None:
-    array = numpy.random.random_sample(size=(238, 318, 4)).astype(numpy.float32)
-    for format in ("gbrapf32be", "gbrapf32le"):
-        frame = VideoFrame.from_ndarray(array, format=format)
-        assert frame.width == 318 and frame.height == 238
-        assert frame.format.name == format
-        assertNdarraysEqual(frame.to_ndarray(), array)
-
-
 def test_ndarray_yuv420p() -> None:
     array = numpy.random.randint(0, 256, size=(720, 640), dtype=numpy.uint8)
     frame = VideoFrame.from_ndarray(array, format="yuv420p")
@@ -491,7 +714,7 @@ def test_ndarray_yuv422p10le() -> None:
         frame = VideoFrame.from_ndarray(array, format=format)
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
-        assert format in av.video.frame.supported_np_pix_fmts
+        assert format in supported_np_pix_fmts
 
 
 def test_ndarray_yuv444p16_align() -> None:
@@ -529,28 +752,6 @@ def test_ndarray_yuyv422_align() -> None:
     assertNdarraysEqual(frame.to_ndarray(), array)
 
 
-def test_ndarray_gray16be() -> None:
-    array = numpy.random.randint(0, 65536, size=(480, 640), dtype=numpy.uint16)
-    frame = VideoFrame.from_ndarray(array, format="gray16be")
-    assert frame.width == 640 and frame.height == 480
-    assert frame.format.name == "gray16be"
-    assertNdarraysEqual(frame.to_ndarray(), array)
-
-    # check endianness by examining value of first pixel
-    assertPixelValue16(frame.planes[0], array[0][0], "big")
-
-
-def test_ndarray_gray16le() -> None:
-    array = numpy.random.randint(0, 65536, size=(480, 640), dtype=numpy.uint16)
-    frame = VideoFrame.from_ndarray(array, format="gray16le")
-    assert frame.width == 640 and frame.height == 480
-    assert frame.format.name == "gray16le"
-    assertNdarraysEqual(frame.to_ndarray(), array)
-
-    # check endianness by examining value of first pixel
-    assertPixelValue16(frame.planes[0], array[0][0], "little")
-
-
 def test_ndarray_rgb48be() -> None:
     array = numpy.random.randint(0, 65536, size=(480, 640, 3), dtype=numpy.uint16)
     frame = VideoFrame.from_ndarray(array, format="rgb48be")
@@ -562,6 +763,17 @@ def test_ndarray_rgb48be() -> None:
     assertPixelValue16(frame.planes[0], array[0][0][0], "big")
 
 
+def test_ndarray_bgr48be() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640, 3), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="bgr48be")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "bgr48be"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining blue value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0][0], "big")
+
+
 def test_ndarray_rgb48le() -> None:
     array = numpy.random.randint(0, 65536, size=(480, 640, 3), dtype=numpy.uint16)
     frame = VideoFrame.from_ndarray(array, format="rgb48le")
@@ -573,6 +785,17 @@ def test_ndarray_rgb48le() -> None:
     assertPixelValue16(frame.planes[0], array[0][0][0], "little")
 
 
+def test_ndarray_bgr48le() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640, 3), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="bgr48le")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "bgr48le"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining blue value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0][0], "little")
+
+
 def test_ndarray_rgb48le_align() -> None:
     array = numpy.random.randint(0, 65536, size=(238, 318, 3), dtype=numpy.uint16)
     frame = VideoFrame.from_ndarray(array, format="rgb48le")
@@ -584,6 +807,17 @@ def test_ndarray_rgb48le_align() -> None:
     assertPixelValue16(frame.planes[0], array[0][0][0], "little")
 
 
+def test_ndarray_bgr48le_align() -> None:
+    array = numpy.random.randint(0, 65536, size=(238, 318, 3), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="bgr48le")
+    assert frame.width == 318 and frame.height == 238
+    assert frame.format.name == "bgr48le"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining blue value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0][0], "little")
+
+
 def test_ndarray_rgba64be() -> None:
     array = numpy.random.randint(0, 65536, size=(480, 640, 4), dtype=numpy.uint16)
     frame = VideoFrame.from_ndarray(array, format="rgba64be")
@@ -595,6 +829,17 @@ def test_ndarray_rgba64be() -> None:
     assertPixelValue16(frame.planes[0], array[0][0][0], "big")
 
 
+def test_ndarray_bgra64be() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640, 4), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="bgra64be")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "bgra64be"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining blue value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0][0], "big")
+
+
 def test_ndarray_rgba64le() -> None:
     array = numpy.random.randint(0, 65536, size=(480, 640, 4), dtype=numpy.uint16)
     frame = VideoFrame.from_ndarray(array, format="rgba64le")
@@ -606,6 +851,35 @@ def test_ndarray_rgba64le() -> None:
     assertPixelValue16(frame.planes[0], array[0][0][0], "little")
 
 
+def test_ndarray_bgra64le() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640, 4), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="bgra64le")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "bgra64le"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining blue value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0][0], "little")
+
+
+def test_ndarray_rgbaf16() -> None:
+    array = numpy.random.random_sample(size=(480, 640, 4)).astype(numpy.float16)
+    for format in ("rgbaf16be", "rgbaf16le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_rgbaf32() -> None:
+    array = numpy.random.random_sample(size=(480, 640, 4)).astype(numpy.float32)
+    for format in ("rgbaf32be", "rgbaf32le"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
 def test_ndarray_rgb8() -> None:
     array = numpy.random.randint(0, 256, size=(480, 640), dtype=numpy.uint8)
     frame = VideoFrame.from_ndarray(array, format="rgb8")
@@ -628,7 +902,7 @@ def test_ndarray_pal8():
     frame = VideoFrame.from_ndarray((array, palette), format="pal8")
     assert frame.width == 640 and frame.height == 480
     assert frame.format.name == "pal8"
-    assert frame.format.name in av.video.frame.supported_np_pix_fmts
+    assert frame.format.name in supported_np_pix_fmts
     returned = frame.to_ndarray()
     assert type(returned) is tuple and len(returned) == 2
     assertNdarraysEqual(returned[0], array)
@@ -640,7 +914,7 @@ def test_ndarray_nv12() -> None:
     frame = VideoFrame.from_ndarray(array, format="nv12")
     assert frame.width == 640 and frame.height == 480
     assert frame.format.name == "nv12"
-    assert frame.format.name in av.video.frame.supported_np_pix_fmts
+    assert frame.format.name in supported_np_pix_fmts
     assertNdarraysEqual(frame.to_ndarray(), array)
 
 
@@ -796,6 +1070,25 @@ def test_shares_memory_rgba() -> None:
     assertNdarraysEqual(frame.to_ndarray(), array)
 
 
+def test_shares_memory_bayer8() -> None:
+    for format in ("bayer_rggb8", "bayer_bggr8", "bayer_grbg8", "bayer_gbrg8"):
+        array = numpy.random.randint(0, 256, size=(357, 318), dtype=numpy.uint8)
+        frame = VideoFrame.from_numpy_buffer(array, format)
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+        array[...] = numpy.random.randint(0, 256, size=(357, 318), dtype=numpy.uint8)
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+        array = numpy.random.randint(0, 256, size=(357, 318), dtype=numpy.uint8)
+        array = array[:, :300]
+        assert not array.data.c_contiguous
+        frame = VideoFrame.from_numpy_buffer(array, format)
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+        array[...] = numpy.random.randint(0, 256, size=array.shape, dtype=numpy.uint8)
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
 def test_shares_memory_yuv420p() -> None:
     array = numpy.random.randint(0, 256, size=(512 * 6 // 4, 256), dtype=numpy.uint8)
     frame = VideoFrame.from_numpy_buffer(array, "yuv420p")
